Package: switchSelection
Type: Package
Title: Endogenous Switching and Sample Selection Regression Models
Version: 2.1.0
Date: 2026-04-15
Authors@R: c(person("Bogdan", "Potanin", email = "bogdanpotanin@gmail.com", role = c("aut", "cre", "ctb")), person("Sofiia", "Dolgikh", email = "sidolgikh@gmail.com", role = c("ctb")))
Description: Estimate the parameters of multivariate endogenous switching and sample selection models using methods described in Newey (2009) <doi:10.1111/j.1368-423X.2008.00263.x>, E. Kossova, B. Potanin (2018) <https://ideas.repec.org/a/ris/apltrx/0346.html>, E. Kossova, L. Kupriianova, B. Potanin (2020) <https://ideas.repec.org/a/ris/apltrx/0391.html> and E. Kossova, B. Potanin (2022) <https://ideas.repec.org/a/ris/apltrx/0455.html>. 
License: GPL (>= 2)
Imports: Rcpp (>= 1.1.1), hpa (>= 1.3.4), mnorm (>= 1.2.3), gena (>=
        1.0.1), dplyr (>= 1.1.4), methods
LinkingTo: Rcpp, RcppArmadillo, hpa, mnorm
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-04-15 15:00:33 UTC; Bogdan
Author: Bogdan Potanin [aut, cre, ctb],
  Sofiia Dolgikh [ctb]
Maintainer: Bogdan Potanin <bogdanpotanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-15 15:30:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-17 07:42:37 UTC; unix
Archs: switchSelection.so.dSYM
